@@ -2,9 +2,12 @@
 
 __author__ = """Yossi Mosbacher"""
 __email__ = "joe.mosbacher@gmail.com"
-__version__ = "0.1.10"
+__version__ = "0.1.12"
 
-from .indexes import *
+from . import schema, indexes, utils
+
+from .indexes import Index, InterpolatingIndex, IntervalIndex
+from .types import Interval, IntegerInterval, TimeInterval
 from .rframe import RemoteFrame
 from .schema import BaseSchema
 from .rest_client import BaseRestClient, RestClient
