@@ -7,7 +7,7 @@ from loguru import logger
 class BaseDataQuery(ABC):
 
     @abstractmethod
-    def execute(self, limit=None, offset=None):
+    def execute(self, limit=None, offset=None): # pragma: no cover
         pass
 
     def unique(self, fields: Union[str, List[str]]):
@@ -74,7 +74,7 @@ class DatasourceInterface(ABC):
         )
 
     @classmethod
-    def from_url(cls, url: str, *args, **kwargs):
+    def from_url(cls, url: str, *args, **kwargs): 
         raise NotImplementedError
 
     @abstractmethod
@@ -91,4 +91,8 @@ class DatasourceInterface(ABC):
         raise NotImplementedError
 
     def update_many(self, docs: list) -> list:
-        return [self.update(doc) for doc in docs]
\ No newline at end of file
+        return [self.update(doc) for doc in docs]
+
+    def delete(self, doc):
+        raise NotImplementedError
+    
\ No newline at end of file
