@@ -6,7 +6,7 @@ from loguru import logger
 import pandas as pd
 from pydantic import BaseModel
 
-from rframe.indexes.types import Interval
+from ..types import Interval
 
 from ..indexes import Index, InterpolatingIndex, IntervalIndex, MultiIndex
 from ..utils import singledispatchmethod
@@ -100,7 +100,8 @@ try:
                     }
                     })
                     
-                results[field] = [doc['first'] for doc in self.collection.aggregate(pipeline, allowDiskUse=True)]
+                results[field] = [doc['first'] for doc in 
+                                  self.collection.aggregate(pipeline, allowDiskUse=True)]
             if len(fields) == 1:
                 return results[fields[0]]
             return results
@@ -116,8 +117,10 @@ try:
                 })
                 pipeline.append({"$limit": 1})
                 pipeline.append({"$project": { "_id": 0}})
-                results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
-
+                try:
+                    results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
+                except StopIteration:
+                    results[field] = None
             if len(fields) == 1:
                 return results[fields[0]]
             return results
@@ -133,7 +136,10 @@ try:
                 })
                 pipeline.append({"$limit": 1})
                 pipeline.append({"$project": { "_id": 0}})
-                results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
+                try:
+                    results[field] = next(self.collection.aggregate(pipeline, allowDiskUse=True))[field]
+                except StopIteration:
+                    results[field] = None
 
             if len(fields) == 1:
                 return results[fields[0]]
@@ -142,7 +148,10 @@ try:
         def count(self):
             pipeline = list(self.pipeline)
             pipeline.append({"$count": "count"})
-            result = next(self.collection.aggregate(pipeline, allowDiskUse=True))
+            try:
+                result = next(self.collection.aggregate(pipeline, allowDiskUse=True))
+            except StopIteration:
+                return 0
             return result.get('count', 0)
 
         def logical_and(self, other):
@@ -367,6 +376,10 @@ try:
         def ensure_index(self, names, order=pymongo.ASCENDING):
             self.source.ensure_index([(name, order) for name in names])
 
+        def delete(self, doc):
+            return self.source.delete_one(doc.index_labels)
+
+
 except ImportError:
 
     class MongoInterface:
@@ -452,7 +465,7 @@ def mongo_after_query(name, value, limit=1):
     if isinstance(limit, list):
         return mongo_grouped_after_query(name, value, limit)
     return [
-        {"$match": {f"{name}": {"$gte": value}}},
+        {"$match": {f"{name}": {"$gt": value}}},
         {"$sort": {f"{name}": 1}},
         {"$limit": limit},
     ]
@@ -474,7 +487,7 @@ def mongo_grouped_before_query(name, value, groups):
 
 def mongo_grouped_after_query(name, value, groups):
     return [
-        {"$match": {f"{name}": {"$gte": value}}},
+        {"$match": {f"{name}": {"$gt": value}}},
         {"$sort": {f"{name}": 1}},
         {"$group": {"_id": [f"${grp}" for grp in groups], "doc": {"$first": "$$ROOT"}}},
         {
