@@ -102,4 +102,8 @@ class RestInterface(DatasourceInterface):
         logger.debug(f'REST api backend inserting document {doc}')
         return self.source.insert(doc)
 
-    update = insert
\ No newline at end of file
+    update = insert
+
+    def delete(self, doc):
+        logger.debug(f'REST api backend deleting document {doc}')
+        return self.source.delete(doc)
\ No newline at end of file
