@@ -12,11 +12,11 @@ from .utils import jsonable
 class BaseRestClient(ABC):
 
     @abstractmethod
-    def query(self, **params):
+    def query(self, **params): # pragma: no cover
         pass
     
     @abstractmethod
-    def insert(self, data):
+    def insert(self, data): # pragma: no cover
         pass
 
     def unique(self, field):
@@ -61,6 +61,15 @@ class RestClient(BaseRestClient):
         with logger.catch():
             r.raise_for_status()
         return r.json()
+    
+    update = insert
+
+    def delete(self, doc):
+        data = doc.json()
+        r = self.client.delete(self.url, headers=self.headers, data=data)
+        with logger.catch():
+            r.raise_for_status()
+        return r.json()
 
     def unique(self, fields: List[str] = None, **labels):
         if not isinstance(fields, list):
@@ -141,4 +150,4 @@ class RestClient(BaseRestClient):
 
         if len(fields) == 1:
             return data[fields[0]]
-        return data
\ No newline at end of file
+        return data
