@@ -1,24 +1,32 @@
-from __future__ import annotations
 import inspect
-from numpy import isin
+import json
 
 import pandas as pd
 from pydantic import BaseModel
 from pydantic.fields import FieldInfo, ModelField
 from typing import Dict, List, Mapping, Optional, Union
 
+from rframe.utils import are_equal
+
 from .indexes import BaseIndex, Index, MultiIndex
 from .interfaces import get_interface
 
 
-class InsertionError(Exception):
+class EditError(Exception):
     pass
 
-class UpdateError(Exception):
+class InsertionError(EditError):
     pass
 
+class UpdateError(EditError):
+    pass
+
+class DeletionError(EditError):
+    pass
 
 class BaseSchema(BaseModel):
+    class Config:
+        validate_assignment = True
 
     @classmethod
     def default_datasource(cls):
@@ -267,17 +275,40 @@ class BaseSchema(BaseModel):
         interface = get_interface(datasource, **kwargs)
         existing = self.find(datasource, **self.index_labels)
         if not existing:
-            self.__pre_insert(datasource)
-            return interface.insert(self)
+            # No documents found, insert new
+            try:
+                self.__pre_insert(datasource)
+                interface.insert(self)
+            except Exception as e:
+                self.__post_insert(datasource, exception=e)
+                raise  e
+            self.__post_insert(datasource)
+
         elif len(existing) == 1:
-            existing[0].__pre_update(datasource, self)
-            return interface.update(self)
+            # Single document found, update
+            try:
+                existing[0].__pre_update(datasource, self)
+                interface.update(self)
+            except Exception as e:
+                existing[0].__post_update(datasource, self, exception=e)
+                raise e
+            existing[0].__post_update(datasource, self)
         else:
-            raise InsertionError('Multiple documents match document '
+            # Multiple documents found, raise exception
+            raise UpdateError('Multiple documents match document '
                                  f'index ({self.index_labels}). '
                                  'Multiple update is not supported.')
-            
-        
+
+    def delete(self, datasource=None, **kwargs):
+        if datasource is None:
+            datasource = self.default_datasource()
+        interface = get_interface(datasource, **kwargs)
+        try:
+            self.__pre_delete(datasource)
+            interface.delete(self)
+        except Exception as e:
+            self.__post_delete(datasource, exception=e)
+            raise e
 
     def __pre_insert(self, datasource):
         '''This method is called  pre insertion 
@@ -292,6 +323,12 @@ class BaseSchema(BaseModel):
             raise InsertionError(f'Cannot insert new document ({self}).'
                                  f'The schema raised the following exception: {e}')
 
+    def __post_insert(self, datasource, exception=None):
+        '''This method is called post insertion 
+        runs the schemas post insertion hook and returns
+        '''
+        self.post_insert(datasource, exception)
+       
 
     def __pre_update(self, datasource, new):
         '''This method is called if new.save(datasource)
@@ -300,9 +337,9 @@ class BaseSchema(BaseModel):
 
         raises an UpdateError if user defined checks fail.
         '''
-        if not self.same_index(new):
-            raise UpdateError(f'Cannot update document ({self}) with {new}. '
-                              f'The index labels do not match.')
+        # if not self.same_index(new):
+        #     raise UpdateError(f'Cannot update document ({self}) with {new}. '
+        #                       f'The index labels do not match.')
         try:
             self.pre_update(datasource, new=new)
         except Exception as e:
@@ -310,13 +347,45 @@ class BaseSchema(BaseModel):
                               f"with new instance ({new}), the schema "
                               f"raised the following exception: {e}")
 
+    def __post_update(self, datasource, new, exception=None):
+        '''This method is called after updates 
+        runs the schemas post update hook and returns
+        '''
+        self.post_update(datasource, new, exception)
+
+    def __pre_delete(self, datasource):
+        '''This method is called pre deletion 
+        if self.delete(datasource) was called and a query on datasource
+        with self.index_labels returned this document.
+
+        raises an DeletionError if user defined checks fail.
+        '''
+        try:
+            self.pre_delete(datasource)
+        except Exception as e:
+            raise DeletionError(f'Cannot delete document ({self}).'
+                                 f'The schema raised the following exception: {e}')
+
+    def __post_delete(self, datasource, exception=None):
+        '''This method is called post deletion 
+        runs the schemas post deletion hook and returns
+        '''
+        self.post_delete(datasource, exception)
+
     def pre_insert(self, datasource):
-        '''User defined checks to perform
+        '''Pre insert hook for user 
+        defined checks to perform
         prior to document insertion.
         Should raise an exception if insertion
         is disallowed.
         '''
         pass
+
+    def post_insert(self, datasource, exception=None):
+        '''User defined hook to perform
+        after document insertion.
+        '''
+        pass
     
     def pre_update(self, datasource, new):
         '''User defined checks to perform
@@ -326,19 +395,45 @@ class BaseSchema(BaseModel):
         '''
         pass
 
+    def post_update(self, datasource, new, exception=None):
+        '''User defined hook to perform
+        after document updates.
+        '''
+        pass
+    
+    def pre_delete(self, datasource):
+        '''User defined checks to perform
+        prior to document deletion.
+        Should raise an exception if deletion
+        is disallowed.
+        '''
+        pass
+
+    def post_delete(self, datasource, exception=None):
+        '''User defined hook to perform
+        after document deletion.
+        '''
+        pass
+
+    def delete(self, datasource=None):
+        if datasource is None:
+            datasource = self.default_datasource()
+        interface = get_interface(datasource)
+        return interface.delete(self)
+
     def same_values(self, other):
         if other is None:
             return False
         if not isinstance(other, BaseSchema):
             return False
-        return self.column_values == other.column_values
+        return are_equal(self.column_values, other.column_values)
 
     def same_index(self, other):
         if other is None:
             return False
         if not isinstance(other, BaseSchema):
             return False
-        return self.index_labels == other.index_labels
+        return are_equal(self.index_labels, other.index_labels)
 
     def pandas_dict(self):
         data = self.dict()
@@ -355,10 +450,35 @@ class BaseSchema(BaseModel):
             indexes = indexes[0]
         return pd.DataFrame().reindex(columns=columns).set_index(indexes)
 
-
     def to_pandas(self):
         index_fields = list(self.get_index_fields())
         if len(index_fields) == 1:
             index_fields = index_fields[0]
         df = pd.DataFrame([self.pandas_dict()])
         return df.set_index(index_fields)
+
+    def jsonable(self):
+        return json.loads(self.json())
+
+    @property
+    def raw_index_labels(self):
+        return tuple(getattr(self, k) for k in self.get_index_fields())
+
+    def __lt__(self, other: 'BaseSchema'):
+        return self.raw_index_labels < other.raw_index_labels
+
+    def __le__(self, other: 'BaseSchema'):
+        if not isinstance(other, BaseSchema):
+            raise TypeError('')
+        return self.raw_index_labels <= other.raw_index_labels
+
+    def __eq__(self, other: 'BaseSchema'):
+        if not isinstance(other, BaseSchema):
+            False
+        return are_equal(self.dict, other.dict())
+
+    def __gt__(self, other: 'BaseSchema'):
+        return self.raw_index_labels > other.raw_index_labels
+
+    def __ge__(self, other: 'BaseSchema'):
+        return self.raw_index_labels >= other.raw_index_labels
